import 'dart:async';
import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;
import 'package:better_client/api_response.dart';

class BetterClient {
  BetterClient({@required this.baseUrl, this.debug, this.headers});

  String baseUrl;
  bool debug = false;
  Map<String, String> headers = {};
  List<int> successCodes = [200, 201, 202];

  Future<dynamic> get(
      {String path, Function fromJson, bool debug, Function parseList}) async {
    try {
      Uri uri = Uri.parse(baseUrl + path);
      http.Response response = await http.get(uri, headers: headers);
      return loadResponse(
          response: response, fromJson: fromJson, parseList: parseList);
    } catch (e) {
      if ((debug != null && debug) || this.debug) printWrapped(e.toString());
      return ApiResponse(success: false, errors: e);
    }
  }

  Future<dynamic> post(
      {String path,
      dynamic body,
      Function fromJson,
      bool debug,
      Function parseList}) async {
    try {
      Uri uri = Uri.parse(baseUrl + path);
      http.Response response =
          await http.post(uri, headers: headers, body: body);
      return loadResponse(
          response: response, fromJson: fromJson, parseList: parseList);
    } catch (e) {
      if ((debug != null && debug) || this.debug) printWrapped(e.toString());
      return ApiResponse(success: false, errors: e);
    }
  }

  Future<dynamic> delete(
      {String path, Function fromJson, bool debug, Function parseList}) async {
    try {
      Uri uri = Uri.parse(baseUrl + path);
      http.Response response = await http.delete(uri, headers: headers);
      return loadResponse(
          response: response, fromJson: fromJson, parseList: parseList);
    } catch (e) {
      if ((debug != null && debug) || this.debug) printWrapped(e.toString());
      return ApiResponse(success: false, errors: e);
    }
  }

  Future<dynamic> put(
      {String path,
      Function fromJson,
      bool debug,
      dynamic body,
      Function parseList}) async {
    try {
      Uri uri = Uri.parse(baseUrl + path);
      http.Response response =
          await http.put(uri, headers: headers, body: body);
      return loadResponse(
          response: response, fromJson: fromJson, parseList: parseList);
    } catch (e) {
      if ((debug != null && debug) || this.debug) printWrapped(e.toString());
      return ApiResponse(success: false, errors: e);
    }
  }

  Future<dynamic> patch(
      {String path,
      Function fromJson,
      bool debug,
      dynamic body,
      Function parseList}) async {
    try {
      Uri uri = Uri.parse(baseUrl + path);
      http.Response response =
          await http.patch(uri, headers: headers, body: body);
      return loadResponse(
          response: response, fromJson: fromJson, parseList: parseList);
    } catch (e) {
      if ((debug != null && debug) || this.debug) printWrapped(e.toString());
      return ApiResponse(success: false, errors: e);
    }
  }

  dynamic _parseData({dynamic data, Function fromJson, Function parseList}) {
    if (data == null || fromJson == null) return;
    data = jsonDecode(data);
    var list = [];
    if (data is List) {
      data.forEach((el) {
        var model = fromJson(el);
        list.add(model);
      });
      return parseList(list);
    } else {
      return fromJson(data);
    }
  }

  ApiResponse loadResponse(
      {http.Response response, Function fromJson, Function parseList}) {
    if ((debug != null && debug) || this.debug)
      printWrapped(response.statusCode.toString());
    if (successCodes.contains(response.statusCode)) {
      if ((debug != null && debug) || this.debug) printWrapped(response.body);
      return ApiResponse(
        success: true,
        body: _parseData(
            data: response.body, fromJson: fromJson, parseList: parseList),
      );
    } else {
      if ((debug != null && debug) || this.debug) printWrapped(response.body);
      return ApiResponse(success: false, errors: response.body);
    }
  }

  void printWrapped(String text) {
    final pattern = RegExp('.{1,800}'); // 800 is the size of each chunk
    pattern.allMatches(text).forEach((match) => print(match.group(0)));
  }
}
